@@ -16,6 +16,7 @@
 #include "nss.h"
 #include "secport.h"
 #include "secpkcs5.h"
+#include "sechash.h"
 #include "certdb.h"
 
 #define PKCS12_IN_BUFFER_SIZE 200
@@ -23,11 +24,12 @@
 static char *progName;
 PRBool pk12_debugging = PR_FALSE;
 PRBool dumpRawFile;
+static PRBool pk12uForceUnicode;
 
 PRIntn pk12uErrno = 0;
 
 static void
-Usage(char *progName)
+Usage()
 {
 #define FPS PR_fprintf(PR_STDERR,
     FPS "Usage:	 %s -i importfile [-d certdir] [-P dbprefix] [-h tokenname]\n",
@@ -42,7 +44,7 @@ Usage(char *progName)
 
     FPS "Usage:	 %s -o exportfile -n certname [-d certdir] [-P dbprefix]\n",
 		progName);
-    FPS "\t\t [-c key_cipher] [-C cert_cipher]\n"
+    FPS "\t\t [-c key_cipher] [-C cert_cipher] [-M mac_alg]\n"
         "\t\t [-m | --key_len keyLen] [--cert_key_len certKeyLen] [-v]\n");
     FPS "\t\t [-k slotpwfile | -K slotpw]\n"
         "\t\t [-w p12filepwfile | -W p12filepw]\n");
@@ -357,6 +359,7 @@ p12U_ReadPKCS12File(SECItem *uniPwp, char *in_file, PK11SlotInfo *slot,
     SECItem p12file = { 0 };
     SECStatus rv = SECFailure;
     PRBool swapUnicode = PR_FALSE;
+    PRBool forceUnicode = pk12uForceUnicode;
     PRBool trypw;
     int error;
 
@@ -424,6 +427,18 @@ p12U_ReadPKCS12File(SECItem *uniPwp, char *in_file, PK11SlotInfo *slot,
                 SEC_PKCS12DecoderFinish(p12dcx);
                 uniPwp->len = 0;
                 trypw = PR_TRUE;
+            } else if (forceUnicode == pk12uForceUnicode) {
+                /* try again with a different password encoding */
+                forceUnicode = !pk12uForceUnicode;
+                rv = NSS_OptionSet(__NSS_PKCS12_DECODE_FORCE_UNICODE,
+                                   forceUnicode);
+                if (rv != SECSuccess) {
+                    SECU_PrintError(progName, "PKCS12 decoding failed to set option");
+                    pk12uErrno = PK12UERR_DECODEVERIFY;
+                    break;
+                }
+                SEC_PKCS12DecoderFinish(p12dcx);
+                trypw = PR_TRUE;
             } else {
                 SECU_PrintError(progName, "PKCS12 decode not verified");
                 pk12uErrno = PK12UERR_DECODEVERIFY;
@@ -431,6 +446,15 @@ p12U_ReadPKCS12File(SECItem *uniPwp, char *in_file, PK11SlotInfo *slot,
             }
         }
     } while (trypw == PR_TRUE);
+
+    /* revert the option setting */
+    if (forceUnicode != pk12uForceUnicode) {
+        rv = NSS_OptionSet(__NSS_PKCS12_DECODE_FORCE_UNICODE, pk12uForceUnicode);
+        if (rv != SECSuccess) {
+            SECU_PrintError(progName, "PKCS12 decoding failed to set option");
+            pk12uErrno = PK12UERR_DECODEVERIFY;
+        }
+    }
 /* rv has been set at this point */
 
 done:
@@ -470,6 +494,8 @@ P12U_ImportPKCS12Object(char *in_file, PK11SlotInfo *slot,
 {
     SEC_PKCS12DecoderContext *p12dcx = NULL;
     SECItem uniPwitem = { 0 };
+    PRBool forceUnicode = pk12uForceUnicode;
+    PRBool trypw;
     SECStatus rv = SECFailure;
 
     rv = P12U_InitSlot(slot, slotPw);
@@ -480,31 +506,62 @@ P12U_ImportPKCS12Object(char *in_file, PK11SlotInfo *slot,
         return rv;
     }
 
-    rv = SECFailure;
-    p12dcx = p12U_ReadPKCS12File(&uniPwitem, in_file, slot, slotPw, p12FilePw);
+    do {
+        trypw = PR_FALSE; /* normally we do this once */
+        rv = SECFailure;
+        p12dcx = p12U_ReadPKCS12File(&uniPwitem, in_file, slot, slotPw, p12FilePw);
 
-    if (p12dcx == NULL) {
-        goto loser;
-    }
-
-    /* make sure the bags are okey dokey -- nicknames correct, etc. */
-    rv = SEC_PKCS12DecoderValidateBags(p12dcx, P12U_NicknameCollisionCallback);
-    if (rv != SECSuccess) {
-        if (PORT_GetError() == SEC_ERROR_PKCS12_DUPLICATE_DATA) {
-            pk12uErrno = PK12UERR_CERTALREADYEXISTS;
-        } else {
-            pk12uErrno = PK12UERR_DECODEVALIBAGS;
+        if (p12dcx == NULL) {
+            goto loser;
         }
-        SECU_PrintError(progName, "PKCS12 decode validate bags failed");
-        goto loser;
-    }
 
-    /* stuff 'em in */
-    rv = SEC_PKCS12DecoderImportBags(p12dcx);
-    if (rv != SECSuccess) {
-        SECU_PrintError(progName, "PKCS12 decode import bags failed");
-        pk12uErrno = PK12UERR_DECODEIMPTBAGS;
-        goto loser;
+        /* make sure the bags are okey dokey -- nicknames correct, etc. */
+        rv = SEC_PKCS12DecoderValidateBags(p12dcx, P12U_NicknameCollisionCallback);
+        if (rv != SECSuccess) {
+            if (PORT_GetError() == SEC_ERROR_PKCS12_DUPLICATE_DATA) {
+                pk12uErrno = PK12UERR_CERTALREADYEXISTS;
+            } else {
+                pk12uErrno = PK12UERR_DECODEVALIBAGS;
+            }
+            SECU_PrintError(progName, "PKCS12 decode validate bags failed");
+            goto loser;
+        }
+
+        /* stuff 'em in */
+        if (forceUnicode != pk12uForceUnicode) {
+            rv = NSS_OptionSet(__NSS_PKCS12_DECODE_FORCE_UNICODE,
+                               forceUnicode);
+            if (rv != SECSuccess) {
+                SECU_PrintError(progName, "PKCS12 decode set option failed");
+                pk12uErrno = PK12UERR_DECODEIMPTBAGS;
+                goto loser;
+            }
+        }
+        rv = SEC_PKCS12DecoderImportBags(p12dcx);
+        if (rv != SECSuccess) {
+            if (PR_GetError() == SEC_ERROR_PKCS12_UNABLE_TO_IMPORT_KEY &&
+                forceUnicode == pk12uForceUnicode) {
+                /* try again with a different password encoding */
+                forceUnicode = !pk12uForceUnicode;
+                SEC_PKCS12DecoderFinish(p12dcx);
+                SECITEM_ZfreeItem(&uniPwitem, PR_FALSE);
+                trypw = PR_TRUE;
+            } else {
+                SECU_PrintError(progName, "PKCS12 decode import bags failed");
+                pk12uErrno = PK12UERR_DECODEIMPTBAGS;
+                goto loser;
+            }
+        }
+    } while (trypw);
+
+    /* revert the option setting */
+    if (forceUnicode != pk12uForceUnicode) {
+        rv = NSS_OptionSet(__NSS_PKCS12_DECODE_FORCE_UNICODE, pk12uForceUnicode);
+        if (rv != SECSuccess) {
+            SECU_PrintError(progName, "PKCS12 decode set option failed");
+            pk12uErrno = PK12UERR_DECODEIMPTBAGS;
+            goto loser;
+        }
     }
 
     fprintf(stdout, "%s: PKCS12 IMPORT SUCCESSFUL\n", progName);
@@ -569,7 +626,7 @@ p12u_WriteToExportFile(void *arg, const char *buf, unsigned long len)
 
 void
 P12U_ExportPKCS12Object(char *nn, char *outfile, PK11SlotInfo *inSlot,
-                        SECOidTag cipher, SECOidTag certCipher,
+                        SECOidTag cipher, SECOidTag certCipher, SECOidTag hash,
                         secuPWData *slotPw, secuPWData *p12FilePw)
 {
     SEC_PKCS12ExportContext *p12ecx = NULL;
@@ -588,6 +645,7 @@ P12U_ExportPKCS12Object(char *nn, char *outfile, PK11SlotInfo *inSlot,
     }
     certlist = PK11_FindCertsFromNickname(nn, slotPw);
     if (!certlist) {
+        PORT_SetError(SEC_ERROR_UNKNOWN_CERT);
         SECU_PrintError(progName, "find user certs from nickname failed");
         pk12uErrno = PK12UERR_FINDCERTBYNN;
         return;
@@ -615,11 +673,7 @@ P12U_ExportPKCS12Object(char *nn, char *outfile, PK11SlotInfo *inSlot,
     }
 
     if (certlist) {
-        CERTCertificate *cert = NULL;
-        node = CERT_LIST_HEAD(certlist);
-        if (node) {
-            cert = node->cert;
-        }
+        CERTCertificate *cert = CERT_LIST_HEAD(certlist)->cert;
         if (cert) {
             slot = cert->slot; /* use the slot from the first matching
                 certificate to create the context . This is for keygen */
@@ -637,7 +691,7 @@ P12U_ExportPKCS12Object(char *nn, char *outfile, PK11SlotInfo *inSlot,
         goto loser;
     }
 
-    if (SEC_PKCS12AddPasswordIntegrity(p12ecx, pwitem, SEC_OID_SHA1) !=
+    if (SEC_PKCS12AddPasswordIntegrity(p12ecx, pwitem, hash) !=
         SECSuccess) {
         SECU_PrintError(progName, "PKCS12 add password integrity failed");
         pk12uErrno = PK12UERR_PK12ADDPWDINTEG;
@@ -669,8 +723,8 @@ P12U_ExportPKCS12Object(char *nn, char *outfile, PK11SlotInfo *inSlot,
         }
 
         if (SEC_PKCS12AddCertAndKey(p12ecx, certSafe, NULL, cert,
-                                    CERT_GetDefaultCertDB(), keySafe, NULL, PR_TRUE, pwitem, cipher) !=
-            SECSuccess) {
+                                    CERT_GetDefaultCertDB(), keySafe, NULL,
+                                    PR_TRUE, pwitem, cipher) != SECSuccess) {
             SECU_PrintError(progName, "add cert and key failed");
             pk12uErrno = PK12UERR_ADDCERTKEY;
             goto loser;
@@ -752,8 +806,7 @@ P12U_ListPKCS12File(char *in_file, PK11SlotInfo *slot,
                             PR_Close(fd);
                         }
                     } else if (SECU_PrintSignedData(stdout, dip->der,
-                                                    (dip->hasKey) ?
-                                                                  "(has private key)"
+                                                    (dip->hasKey) ? "(has private key)"
                                                                   : "",
                                                     0, (SECU_PPFunc)SECU_PrintCertificate) !=
                                0) {
@@ -805,25 +858,14 @@ loser:
     return rv;
 }
 
-/*
- * use the oid table description to map a user input string to a particular
- * oid.
- */
 SECOidTag
-PKCS12U_MapCipherFromString(char *cipherString, int keyLen)
+PKCS12U_FindTagFromString(char *cipherString)
 {
     SECOidTag tag;
     SECOidData *oid;
-    SECOidTag cipher;
 
     /* future enhancement: accept dotted oid spec? */
 
-    /* future enhancement: provide 'friendlier' typed in names for
-     * pbe mechanisms.
-     */
-
-    /* look for the oid tag by Description */
-    cipher = SEC_OID_UNKNOWN;
     for (tag = 1; (oid = SECOID_FindOIDByTag(tag)) != NULL; tag++) {
         /* only interested in oids that we actually understand */
         if (oid->mechanism == CKM_INVALID_MECHANISM) {
@@ -832,27 +874,68 @@ PKCS12U_MapCipherFromString(char *cipherString, int keyLen)
         if (PORT_Strcasecmp(oid->desc, cipherString) != 0) {
             continue;
         }
-        /* we found a match... get the PBE version of this
-	 * cipher... */
-        if (!SEC_PKCS5IsAlgorithmPBEAlgTag(tag)) {
-            cipher = SEC_PKCS5GetPBEAlgorithm(tag, keyLen);
-            /* no eqivalent PKCS5/PKCS12 cipher, use the raw
-	     * encryption tag we got and pass it directly in,
-	     * pkcs12 will use the pkcsv5 mechanism */
-            if (cipher == SEC_OID_PKCS5_PBES2) {
-                cipher = tag;
-            } else if (cipher == SEC_OID_PKCS5_PBMAC1) {
-                /* make sure we have not macing ciphers here */
-                cipher = SEC_OID_UNKNOWN;
-            }
-        } else {
+        return tag;
+    }
+    return SEC_OID_UNKNOWN;
+}
+
+/*
+ * use the oid table description to map a user input string to a particular
+ * oid.
+ */
+SECOidTag
+PKCS12U_MapCipherFromString(char *cipherString, int keyLen)
+{
+    SECOidTag tag;
+    SECOidTag cipher;
+
+    /* future enhancement: provide 'friendlier' typed in names for
+     * pbe mechanisms.
+     */
+
+    /* look for the oid tag by Description */
+    tag = PKCS12U_FindTagFromString(cipherString);
+    if (tag == SEC_OID_UNKNOWN) {
+        return tag;
+    }
+
+    cipher = SEC_OID_UNKNOWN;
+    /* we found a match... get the PBE version of this
+     * cipher... */
+    if (!SEC_PKCS5IsAlgorithmPBEAlgTag(tag)) {
+        cipher = SEC_PKCS5GetPBEAlgorithm(tag, keyLen);
+        /* no eqivalent PKCS5/PKCS12 cipher, use the raw
+         * encryption tag we got and pass it directly in,
+         * pkcs12 will use the pkcsv5 mechanism */
+        if (cipher == SEC_OID_PKCS5_PBES2) {
             cipher = tag;
+        } else if (cipher == SEC_OID_PKCS5_PBMAC1) {
+            /* make sure we have not macing ciphers here */
+            cipher = SEC_OID_UNKNOWN;
         }
-        break;
+    } else {
+        cipher = tag;
     }
     return cipher;
 }
 
+SECOidTag
+PKCS12U_MapHashFromString(char *hashString)
+{
+    SECOidTag hashAlg;
+
+    /* look for the oid tag by Description */
+    hashAlg = PKCS12U_FindTagFromString(hashString);
+    if (hashAlg == SEC_OID_UNKNOWN) {
+        return hashAlg;
+    }
+    /* make sure it's a hashing oid */
+    if (HASH_GetHashTypeByOidTag(hashAlg) == HASH_AlgNULL) {
+        return SEC_OID_UNKNOWN;
+    }
+    return hashAlg;
+}
+
 static void
 p12u_EnableAllCiphers()
 {
@@ -862,7 +945,10 @@ p12u_EnableAllCiphers()
     SEC_PKCS12EnableCipher(PKCS12_RC2_CBC_128, 1);
     SEC_PKCS12EnableCipher(PKCS12_DES_56, 1);
     SEC_PKCS12EnableCipher(PKCS12_DES_EDE3_168, 1);
-    SEC_PKCS12SetPreferredCipher(PKCS12_DES_EDE3_168, 1);
+    SEC_PKCS12EnableCipher(PKCS12_AES_CBC_128, 1);
+    SEC_PKCS12EnableCipher(PKCS12_AES_CBC_192, 1);
+    SEC_PKCS12EnableCipher(PKCS12_AES_CBC_256, 1);
+    SEC_PKCS12SetPreferredCipher(PKCS12_AES_CBC_256, 1);
 }
 
 static PRUintn
@@ -908,7 +994,8 @@ enum {
     opt_Cipher,
     opt_CertCipher,
     opt_KeyLength,
-    opt_CertKeyLength
+    opt_CertKeyLength,
+    opt_Mac
 };
 
 static secuCommandFlag pk12util_options[] =
@@ -929,7 +1016,8 @@ static secuCommandFlag pk12util_options[] =
       { /* opt_Cipher	       */ 'c', PR_TRUE, 0, PR_FALSE },
       { /* opt_CertCipher	       */ 'C', PR_TRUE, 0, PR_FALSE },
       { /* opt_KeyLength         */ 'm', PR_TRUE, 0, PR_FALSE, "key_len" },
-      { /* opt_CertKeyLength     */ 0, PR_TRUE, 0, PR_FALSE, "cert_key_len" }
+      { /* opt_CertKeyLength     */ 0, PR_TRUE, 0, PR_FALSE, "cert_key_len" },
+      { /* opt_Mac               */ 'M', PR_TRUE, 0, PR_FALSE, PR_FALSE }
     };
 
 int
@@ -943,12 +1031,13 @@ main(int argc, char **argv)
     char *export_file = NULL;
     char *dbprefix = "";
     SECStatus rv;
-    SECOidTag cipher =
-        SEC_OID_PKCS12_V2_PBE_WITH_SHA1_AND_3KEY_TRIPLE_DES_CBC;
-    SECOidTag certCipher;
+    SECOidTag cipher = SEC_OID_AES_256_CBC;
+    SECOidTag hash = SEC_OID_SHA256;
+    SECOidTag certCipher = SEC_OID_AES_128_CBC;
     int keyLen = 0;
     int certKeyLen = 0;
     secuCommand pk12util;
+    PRInt32 forceUnicode;
 
 #ifdef _CRTDBG_MAP_ALLOC
     _CrtSetDbgFlag(_CRTDBG_ALLOC_MEM_DF | _CRTDBG_LEAK_CHECK_DF);
@@ -965,27 +1054,33 @@ main(int argc, char **argv)
     rv = SECU_ParseCommandLine(argc, argv, progName, &pk12util);
 
     if (rv != SECSuccess)
-        Usage(progName);
+        Usage();
 
     pk12_debugging = pk12util.options[opt_Debug].activated;
 
     if ((pk12util.options[opt_Import].activated +
          pk12util.options[opt_Export].activated +
          pk12util.options[opt_List].activated) != 1) {
-        Usage(progName);
+        Usage();
     }
 
     if (pk12util.options[opt_Export].activated &&
         !pk12util.options[opt_Nickname].activated) {
-        Usage(progName);
+        Usage();
     }
 
+    rv = NSS_OptionGet(__NSS_PKCS12_DECODE_FORCE_UNICODE, &forceUnicode);
+    if (rv != SECSuccess) {
+        SECU_PrintError(progName,
+                        "Failed to get NSS_PKCS12_DECODE_FORCE_UNICODE option");
+        Usage();
+    }
+    pk12uForceUnicode = forceUnicode;
+
     slotname = SECU_GetOptionArg(&pk12util, opt_TokenName);
 
-    import_file = (pk12util.options[opt_List].activated) ?
-                                                         SECU_GetOptionArg(&pk12util, opt_List)
-                                                         :
-                                                         SECU_GetOptionArg(&pk12util, opt_Import);
+    import_file = (pk12util.options[opt_List].activated) ? SECU_GetOptionArg(&pk12util, opt_List)
+                                                         : SECU_GetOptionArg(&pk12util, opt_Import);
     export_file = SECU_GetOptionArg(&pk12util, opt_Export);
 
     if (pk12util.options[opt_P12FilePWFile].activated) {
@@ -1052,8 +1147,6 @@ main(int argc, char **argv)
         }
     }
 
-    certCipher = PK11_IsFIPS() ? SEC_OID_UNKNOWN :
-                               SEC_OID_PKCS12_V2_PBE_WITH_SHA1_AND_40_BIT_RC2_CBC;
     if (pk12util.options[opt_CertCipher].activated) {
         char *cipherString = pk12util.options[opt_CertCipher].arg;
 
@@ -1063,7 +1156,7 @@ main(int argc, char **argv)
             certCipher = PKCS12U_MapCipherFromString(cipherString, certKeyLen);
             /* If the user requested a cipher and we didn't find it, then
 	     * don't just silently not encrypt. */
-            if (cipher == SEC_OID_UNKNOWN) {
+            if (certCipher == SEC_OID_UNKNOWN) {
                 PORT_SetError(SEC_ERROR_INVALID_ALGORITHM);
                 SECU_PrintError(progName, "Algorithm: \"%s\"", cipherString);
                 pk12uErrno = PK12UERR_INVALIDALGORITHM;
@@ -1071,6 +1164,18 @@ main(int argc, char **argv)
             }
         }
     }
+    if (pk12util.options[opt_Mac].activated) {
+        char *hashString = pk12util.options[opt_Mac].arg;
+
+        hash = PKCS12U_MapHashFromString(hashString);
+        /* We don't support creating Mac-less pkcs 12 files */
+        if (hash == SEC_OID_UNKNOWN) {
+            PORT_SetError(SEC_ERROR_INVALID_ALGORITHM);
+            SECU_PrintError(progName, "Algorithm: \"%s\"", hashString);
+            pk12uErrno = PK12UERR_INVALIDALGORITHM;
+            goto done;
+        }
+    }
 
     if (pk12util.options[opt_Import].activated) {
         P12U_ImportPKCS12Object(import_file, slot, &slotPw, &p12FilePw);
@@ -1078,13 +1183,13 @@ main(int argc, char **argv)
     } else if (pk12util.options[opt_Export].activated) {
         P12U_ExportPKCS12Object(pk12util.options[opt_Nickname].arg,
                                 export_file, slot, cipher, certCipher,
-                                &slotPw, &p12FilePw);
+                                hash, &slotPw, &p12FilePw);
 
     } else if (pk12util.options[opt_List].activated) {
         P12U_ListPKCS12File(import_file, slot, &slotPw, &p12FilePw);
 
     } else {
-        Usage(progName);
+        Usage();
         pk12uErrno = PK12UERR_USAGE;
     }
 
