@@ -959,6 +959,34 @@ TEST_P(TlsKeyExchange13, ConnectEcdhePreferenceMismatchHrr) {
   CheckKEXDetails(client_groups, expectedShares, ssl_grp_ec_curve25519);
 }
 
+TEST_P(TlsKeyExchange13, SecondClientHelloPreambleMatches) {
+  server_->SetVersionRange(SSL_LIBRARY_VERSION_TLS_1_3,
+                           SSL_LIBRARY_VERSION_TLS_1_3);
+  client_->SetVersionRange(SSL_LIBRARY_VERSION_TLS_1_1,
+                           SSL_LIBRARY_VERSION_TLS_1_3);
+
+  ConfigureSelfEncrypt();
+  static const std::vector<SSLNamedGroup> client_groups = {
+      ssl_grp_ec_secp384r1, ssl_grp_ec_curve25519};
+  static const std::vector<SSLNamedGroup> server_groups = {
+      ssl_grp_ec_curve25519};
+  client_->ConfigNamedGroups(client_groups);
+  server_->ConfigNamedGroups(server_groups);
+
+  auto ch1 = MakeTlsFilter<ClientHelloPreambleCapture>(client_);
+  StartConnect();
+  client_->Handshake();
+  server_->Handshake();
+
+  MakeNewServer();
+  auto ch2 = MakeTlsFilter<ClientHelloPreambleCapture>(client_);
+  Handshake();
+
+  EXPECT_TRUE(ch1->captured());
+  EXPECT_TRUE(ch2->captured());
+  EXPECT_EQ(ch1->contents(), ch2->contents());
+}
+
 // This should work, but not use HRR because the key share for x25519 was
 // pre-generated by the client.
 TEST_P(TlsKeyExchange13, ConnectEcdhePreferenceMismatchHrrExtraShares) {
@@ -1321,13 +1349,14 @@ TEST_F(TlsConnectStreamTls13, HrrThenTls12SupportedVersions) {
   client_->CheckErrorCode(SSL_ERROR_PROTOCOL_VERSION_ALERT);
 }
 
-INSTANTIATE_TEST_CASE_P(HelloRetryRequestAgentTests, HelloRetryRequestAgentTest,
-                        ::testing::Combine(TlsConnectTestBase::kTlsVariantsAll,
-                                           TlsConnectTestBase::kTlsV13));
+INSTANTIATE_TEST_SUITE_P(HelloRetryRequestAgentTests,
+                         HelloRetryRequestAgentTest,
+                         ::testing::Combine(TlsConnectTestBase::kTlsVariantsAll,
+                                            TlsConnectTestBase::kTlsV13));
 #ifndef NSS_DISABLE_TLS_1_3
-INSTANTIATE_TEST_CASE_P(HelloRetryRequestKeyExchangeTests, TlsKeyExchange13,
-                        ::testing::Combine(TlsConnectTestBase::kTlsVariantsAll,
-                                           TlsConnectTestBase::kTlsV13));
+INSTANTIATE_TEST_SUITE_P(HelloRetryRequestKeyExchangeTests, TlsKeyExchange13,
+                         ::testing::Combine(TlsConnectTestBase::kTlsVariantsAll,
+                                            TlsConnectTestBase::kTlsV13));
 #endif
 
 }  // namespace nss_test
