@@ -11,6 +11,7 @@
 #include <memory>
 #include <set>
 #include <vector>
+#include "pk11pub.h"
 #include "sslt.h"
 #include "sslproto.h"
 #include "test_io.h"
@@ -148,7 +149,7 @@ class TlsRecordHeader : public TlsVersioned {
   const DataBuffer& header() const { return header_; }
 
   bool MaskSequenceNumber();
-  bool MaskSequenceNumber(const DataBuffer& mask);
+  bool MaskSequenceNumber(const DataBuffer& mask_buf);
 
   // Parse the header; return true if successful; body in an outparam if OK.
   bool Parse(bool is_dtls13, uint64_t sequence_number, TlsParser* parser,
@@ -234,6 +235,7 @@ class TlsRecordFilter : public PacketFilter {
     return KEEP;
   }
 
+  bool is_dtls_agent() const;
   bool is_dtls13() const;
   bool is_dtls13_ciphertext(uint8_t ct) const;
   TlsCipherSpec& spec(uint16_t epoch);
@@ -525,6 +527,37 @@ class TlsExtensionReplacer : public TlsExtensionFilter {
   const DataBuffer data_;
 };
 
+class TlsExtensionResizer : public TlsExtensionFilter {
+ public:
+  TlsExtensionResizer(const std::shared_ptr<TlsAgent>& a, uint16_t extension,
+                      size_t length)
+      : TlsExtensionFilter(a), extension_(extension), length_(length) {}
+  PacketFilter::Action FilterExtension(uint16_t extension_type,
+                                       const DataBuffer& input,
+                                       DataBuffer* output) override;
+
+ private:
+  uint16_t extension_;
+  size_t length_;
+};
+
+class TlsExtensionAppender : public TlsHandshakeFilter {
+ public:
+  TlsExtensionAppender(const std::shared_ptr<TlsAgent>& a,
+                       uint8_t handshake_type, uint16_t ext, DataBuffer& data)
+      : TlsHandshakeFilter(a, {handshake_type}), extension_(ext), data_(data) {}
+
+  virtual PacketFilter::Action FilterHandshake(const HandshakeHeader& header,
+                                               const DataBuffer& input,
+                                               DataBuffer* output);
+
+ private:
+  bool UpdateLength(DataBuffer* output, size_t offset, size_t size);
+
+  const uint16_t extension_;
+  const DataBuffer data_;
+};
+
 class TlsExtensionDropper : public TlsExtensionFilter {
  public:
   TlsExtensionDropper(const std::shared_ptr<TlsAgent>& a, uint16_t extension)
@@ -804,6 +837,26 @@ class SelectedCipherSuiteReplacer : public TlsHandshakeFilter {
   uint16_t cipher_suite_;
 };
 
+class ClientHelloPreambleCapture : public TlsHandshakeFilter {
+ public:
+  ClientHelloPreambleCapture(const std::shared_ptr<TlsAgent>& a)
+      : TlsHandshakeFilter(a, {kTlsHandshakeClientHello}),
+        captured_(false),
+        data_() {}
+
+  const DataBuffer& contents() const { return data_; }
+  bool captured() const { return captured_; }
+
+ protected:
+  PacketFilter::Action FilterHandshake(const HandshakeHeader& header,
+                                       const DataBuffer& input,
+                                       DataBuffer* output) override;
+
+ private:
+  bool captured_;
+  DataBuffer data_;
+};
+
 }  // namespace nss_test
 
 #endif
